package it.inaf.oats.ia2.tap.utils;

import it.inaf.oats.ia2.tap.utils.model.ColumnField;
import it.inaf.oats.ia2.tap.utils.model.NullableView;
import it.inaf.oats.ia2.tap.utils.tapschema.Column;
import it.inaf.oats.ia2.tap.utils.tapschema.Schema;
import it.inaf.oats.ia2.tap.utils.tapschema.Table;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.logging.Logger;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.ListView;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.AttributeModifier;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.form.OnChangeAjaxBehavior;
import org.apache.wicket.ajax.markup.html.AjaxFallbackLink;
import org.apache.wicket.behavior.AttributeAppender;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.model.Model;

/**
 *
 * @author Sonia Zorba <zorba at oats.inaf.it>
 */
public final class SchemaPanel extends Panel {

    static final Logger log = Logger.getLogger(SelectSchemataPage.class.getName());

    class TableSelector extends AjaxFallbackLink {

        private final ListItem<String> item;

        public TableSelector(String id, ListItem<String> item) {
            super(id);
            this.item = item;
            add(new Label("tableName", item.getModel()));
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            SchemaPanel.this.selectTable(item.getIndex());
            target.add(SchemaPanel.this);
        }
    }

    class ColumnSelector extends AjaxFallbackLink {

        private final ListItem<String> item;

        public ColumnSelector(String id, ListItem<String> item) {
            super(id);
            this.item = item;
            add(new Label("columnName", item.getModel()));
        }

        @Override
        public void onClick(AjaxRequestTarget target) {
            SchemaPanel.this.selectColumn(item.getIndex());
            target.add(SchemaPanel.this);
        }
    }

    private final Schema schema;

    int selectedTableIndex = 0;
    final ArrayList<String> tablesNames;

    int selectedColumnIndex = 0;
    final ArrayList<String> columnsNames;
    Column selectedColumn = null;

    final NullableView<Column> selectedColumnView;

    public SchemaPanel(String id, Schema schema) {
        super(id);
        this.schema = schema;
        setOutputMarkupPlaceholderTag(true);

        tablesNames = new ArrayList<String>();
        for (String tableName : schema.tablesMap.keySet()) {
            tablesNames.add(tableName);
        }
        add(new ListView<String>("tablesList", tablesNames) {
            @Override
            protected void populateItem(ListItem<String> item) {
                item.add(new TableSelector("tableLink", item));

                if (selectedTableIndex == item.getIndex()) {
                    item.add(new AttributeModifier("class", "active"));
                }
            }
        });

        columnsNames = new ArrayList<String>();
        add(new ListView<String>("columnsList", columnsNames) {
            @Override
            protected void populateItem(ListItem<String> item) {
                item.add(new ColumnSelector("columnLink", item));

                if (selectedColumnIndex == item.getIndex()) {
                    item.add(new AttributeModifier("class", "active"));
                }
            }
        });

        selectedColumnView = new NullableView<Column>("selectedColumn") {

            @Override
            protected void populateItem(ListItem<Column> li) {
                li.add(new ColumnPropertyTextField<String>("utype", ColumnField.UTYPE));
                li.add(new ColumnPropertyTextField<String>("ucd", ColumnField.UCD));
                li.add(new ColumnPropertyTextField<String>("unit", ColumnField.UNIT));
                li.add(new ColumnPropertyTextField<String>("description", ColumnField.DESCRIPTION));
                li.add(new ColumnPropertyTextField<String>("datatype", ColumnField.DATATYPE));
            }
        };

        add(selectedColumnView);

        selectTable(0);
    }

    private Table getSelectedTable() {
        return schema.getTable(tablesNames.get(selectedTableIndex));
    }

    private void selectTable(int index) {
        if (tablesNames.size() > 0) {
            selectedTableIndex = index;

            columnsNames.clear();
            for (String columnName : getSelectedTable().getColumnsMap().keySet()) {
                columnsNames.add(columnName);
            }

            selectColumn(0);
        }
    }

    private void selectColumn(int index) {
        if (columnsNames.size() > 0) {
            selectedColumnIndex = index;
            selectedColumn = getSelectedTable().getColumn(columnsNames.get(index));
            selectedColumnView.setContent(selectedColumn);
        } else {
            selectedColumnView.clear();
            selectedColumn = null;
        }
    }

    class ColumnPropertyTextField<T extends Serializable> extends TextField<T> {

        protected ColumnPropertyTextField(final String id, final ColumnField key) {
            super(id, selectedColumn.getPropertyModel(key));

            add(new AttributeAppender("class", new Model<String>() {
                @Override
                public String getObject() {
                    return selectedColumn.isChanged(key) ? "changed" : "";
                }
            }, " "));

            // TextField class is updated using JavaScript to avoid it will be 
            // rendered again (and lose caret position inside textfield)
            add(new OnChangeAjaxBehavior() {
                @Override
                protected void onUpdate(final AjaxRequestTarget target) {
                    target.appendJavaScript("setChanged('" + id + "'," + selectedColumn.isChanged(key) + ");");
                }
            });
        }
    }
}
